'============================================================
' Projekt        : Record Locking Pro
' Modulname      : GLOBALS.BAS
' Aufgabe        : Deklarationen der DLL-Funktionen (intern)
'                  und darauf basierend wesentlich vereinfach-
'                  ten Routinen zur Datensatzsperrung sowie
'                  globalen Variablen und Konstanten
' Copyright      : Arthur Burda
' Compiler       : Visual Basic 3.0 fr Windows
' Info           : Das Modul enthlt auch Routinen zur Ansteue-
'                  rung der Steuerelemente (Sperren und Entsper-
'                  ren der Editierfelder in einer Form).
'============================================================
' l. nderung am : 10.09.1994
' Version        : 1.01
' Bemerkungen    : Erste Implementierung
'------------------------------------------------------------

Option Explicit

' DLL-Deklarationen

Declare Function int_Init_Locking Lib "REC_LOCK.DLL" Alias "Init_Locking" (ByVal File_Name As String, ByVal User_Name As String, ByVal Lock_Mode As Integer, DOS_Error As Integer) As Integer
Declare Function int_Done_Locking Lib "REC_LOCK.DLL" Alias "Done_Locking" (ByVal File_Name As String, ByVal User_Name As String, DOS_Error As Integer) As Integer
Declare Function int_Login_DateTime Lib "REC_LOCK.DLL" Alias "Login_DateTime" (ByVal File_Name As String, ByVal User_Name As String, Day_ As Integer, Month_ As Integer, Year_ As Integer, Hour_ As Integer, Min_ As Integer, Sec_ As Integer, DOS_Error As Integer) As Integer
Declare Function int_Login_Seconds Lib "REC_LOCK.DLL" Alias "Login_Seconds" (ByVal File_Name As String, ByVal User_Name As String, Seconds As Long, DOS_Error As Integer) As Integer
Declare Function int_What_Lock_Mode Lib "REC_LOCK.DLL" Alias "What_Lock_Mode" (ByVal File_Name As String, ByVal User_Name As String, Lock_Mode As Integer, DOS_Error As Integer) As Integer
Declare Function int_Clear_Table Lib "REC_LOCK.DLL" Alias "Clear_Table" (ByVal File_Name As String, ByVal Table_Name As String, DOS_Error As Integer) As Integer
Declare Function int_Lock_Record Lib "REC_LOCK.DLL" Alias "Lock_Record" (ByVal File_Name As String, ByVal User_Name As String, ByVal Table_Name As String, ByVal Record_ID As Long, ByVal Lock_Timeout As Integer, DOS_Error As Integer) As Integer
Declare Function int_Is_Record_Locked Lib "REC_LOCK.DLL" Alias "Is_Record_Locked" (ByVal File_Name As String, ByVal User_Name As String, ByVal Table_Name As String, ByVal Record_ID As Long, ByVal Lock_User As String, Lock_Day As Integer, Lock_Month As Integer, Lock_Year As Integer, Lock_Hour As Integer, Lock_Min As Integer, Lock_Sec As Integer, Lock_Seconds As Long, Lock_Timeout As Integer, DOS_Error As Integer) As Integer
Declare Function int_Unlock_Record Lib "REC_LOCK.DLL" Alias "Unlock_Record" (ByVal File_Name As String, ByVal Table_Name As String, ByVal Record_ID As Long, DOS_Error As Integer) As Integer

' Status-Konstanten

Global Const st_No_Error = 0            ' kein Fehler
Global Const st_Record_Not_Locked = 0   ' Datensatz ist nicht gesperrt
Global Const st_DOS_Error = 1           ' DOS-Fehler (Details -> globale Variable glb_Last_DOS_Error)
Global Const st_No_File_Name = 2        ' Name der Record-Locking-Verwaltungsdatei fehlt
Global Const st_No_User_Name = 3        ' Username fehlt
Global Const st_Max_Users = 4           ' max. Anzahl User erreicht
Global Const st_File_Destroyed = 5      ' Record-Locking-Verwaltungsdatei ist zerstrt
Global Const st_User_Not_Found = 6      ' User nicht gefunden
Global Const st_No_Table_Name = 7       ' Name der Datenbank-Tabelle bzw. Abfrage fehlt
Global Const st_Max_Tables = 8          ' max. Anzahl Tabellen erreicht
Global Const st_Lock_By_Same = 9        ' Datensatz vom akt. Benutzer gesperrt; keine automatische Entsperrberwachung oder Zeitmarke noch nicht berschritten
Global Const st_Lock_By_Others = 10     ' Datensatz von einem anderem User gesperrt; keine automatische Entsperrberwachung oder Zeitmarke noch nicht berschritten
Global Const st_Lock_By_Same_Out = 11   ' Datensatz vom akt. Benutzer gesperrt; Zeitmarke fr die Sperrung berschritten, Entsperrung wird empfohlen
Global Const st_Lock_By_Others_Out = 12 ' Datensatz von einem anderem User gesperrt; Zeitmarke fr die Sperrung berschritten, Entsperrung wird empfohlen
Global Const st_Max_Records = 13        ' max. Anzahl sperrbarer Datenstze erreicht
Global Const st_Table_Not_Found = 14    ' Datenbank-Tabelle bzw. Abfrage nicht gefunden
Global Const st_Record_Not_Found = 15   ' Datensatz nicht gefunden

' Sperrmethoden-Konstanten

Global Const lock_by_Select = 0 ' Lock-by-Select (Sperren bei Datensatz-Selektion)
Global Const lock_by_Edit = 1   ' Lock-by-Edit (Sperren bei Beginn des Editiervorgangs eines Datensatzes)

' Globale Variablen

Global glb_Last_Status As Integer    ' letzter Status eines DLL-Funktionsaufrufes
Global glb_Last_DOS_Error As Integer ' Code des letzten DOS-Fehlers

Global glb_Lock_File_Name As String  ' Name der Record-Locking-Verwaltungsdatei inkl. Pfad
Global glb_User_Name As String       ' Name des akt. Benutzers in Mehrbenutzer-Umgebung
Global glb_Lock_Mode As Integer      ' Sperrmethode: 0 = Lock-by-Select (Sperren bei Datensatz-Selektion), 1 = Lock-by-Edit (Sperren bei Beginn des Editiervorgangs eines Datensatzes)

'============================================================
' Routine : Center_Form
'============================================================
' Aufgabe : Stellt eine Form zentriert auf dem Bildschirm dar.
' Eingabe : F = Form
' Ausgabe : keine
'------------------------------------------------------------
'
Sub Center_Form (F As Form)

    F.Top = Screen.Height / 2 - F.Height / 2
    F.Left = Screen.Width / 2 - F.Width / 2

End Sub

'============================================================
' Routine : Clear_Table
'============================================================
' Aufgabe : Lscht einen Tabellen/Abfragen-Eintrag aus der Re-
'           cord-Locking-Verwaltungsdatei inkl. aller zugehri-
'           gen gesperrten Datenstze
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
' Ausgabe : keine
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion wird die globale Variable glb_Lock_File_Name
'           benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Sub Clear_Table (Table_Name As String)

    glb_Last_Status = int_Clear_Table(glb_Lock_File_Name, Table_Name, glb_Last_DOS_Error)

End Sub

'============================================================
' Routine : Done_Locking
'============================================================
' Aufgabe : Beendet das Record-Locking-System. Alle vom akt.
'           Benutzer gesperrten Datenstze werden entsperrt.
' Eingabe : keine
' Ausgabe : keine
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Sub Done_Locking ()

    glb_Last_Status = int_Done_Locking(glb_Lock_File_Name, glb_User_Name, glb_Last_DOS_Error)

    If glb_Last_Status = st_No_Error Then
        glb_Lock_File_Name = ""
        glb_User_Name = ""
        glb_Lock_Mode = -1
    End If

End Sub

'============================================================
' Routine : Draw_Form_Frame
'============================================================
' Aufgabe : Zeichnet einen Rahmen um die bergebene Form.
' Eingabe : F           = Form
'           X_Thickness = horizontale Dicke des Rahmens
'           Y_Thickness = vertikale Dicke des Rahmens
' Ausgabe : keine
'------------------------------------------------------------
'
Sub Draw_Form_Frame (F As Form, X_Thickness As Long, Y_Thickness As Long)

    Dim ulx As Long         ' obere linke Ecke in Twips
    Dim uly As Long         ' obere linke Ecke in Twips
    Dim brx As Long         ' untere rechte Ecke in Twips
    Dim bry As Long         ' untere rechte Ecke in Twips
    Dim ulcolor As Long     ' Farbe oben und links
    Dim brcolor As Long     ' Farbe unten und rechts
    Dim stepsize As Integer
    Dim n As Integer

    ulcolor = 16777215
    brcolor = 8421504
    ulx = F.ScaleLeft + X_Thickness
    uly = F.ScaleTop + Y_Thickness
    brx = F.ScaleWidth - X_Thickness
    bry = F.ScaleHeight - Y_Thickness

    F.AutoRedraw = True

    stepsize = 2
    If stepsize > X_Thickness Then X_Thickness = stepsize
    For n = stepsize To X_Thickness Step stepsize
        F.Line (ulx - n, uly - n)-(ulx - n, bry + n), ulcolor
        F.Line (brx + n, uly - n)-(brx + n, bry + n), brcolor
    Next

    stepsize = 2
    If stepsize > Y_Thickness Then Y_Thickness = stepsize
    For n = stepsize To Y_Thickness Step stepsize
        F.Line (ulx - n, uly - n)-(brx + n, uly - n), ulcolor
        F.Line (ulx - n, bry + n)-(brx + n, bry + n), brcolor
    Next

    F.AutoRedraw = False

End Sub

'============================================================
' Routine : Get_Lock_Date
'============================================================
' Aufgabe : Liefert das Sperrdatum eines Datensatzes in Form
'           eines formatierten Strings zurck, falls der Daten-
'           satz gesperrt ist. Ist der Datensatz nicht gesperrt
'           oder ein Fehler aufgetreten, so wird ein leerer
'           String zurckgeliefert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Sperrdatum; Format: "DD.MM.YYYY" bzw. leerer String
' Info    : Eine Sperrung liegt auch vor, wenn die evtl. ein-
'           gestellte Zeitmarke (Lock-Timeout) fr die automa-
'           tische Entsperrberwachung bereits berschritten
'           ist. Dies jedoch kann mit Hilfe der Funktion
'           Is_Record_Locked abgefragt werden.
'           Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_Date (Table_Name As String, Record_ID As Long) As String

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer
    Dim Str_Day As String
    Dim Str_Month As String
    Dim Str_Year As String

    Get_Lock_Date = ""

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Select Case glb_Last_Status
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Str_Day = LTrim$(RTrim$(Str$(Lock_Day)))
            If Lock_Day < 10 Then Str_Day = "0" & Str_Day
            Str_Month = LTrim$(RTrim$(Str$(Lock_Month)))
            If Lock_Month < 10 Then Str_Month = "0" & Str_Month
            Str_Year = LTrim$(RTrim$(Str$(Lock_Year)))
            Get_Lock_Date = Str_Day & "." & Str_Month & "." & Str_Year
    End Select

End Function

'============================================================
' Routine : Get_Lock_Mode
'============================================================
' Aufgabe : Liefert das Sperrmethode, die fr den akt. Benutzer
'           gilt. Ist ein Fehler aufgetreten, so wird der Wert
'           -1 zurckgeliefert.
' Eingabe : keine
' Ausgabe : keine
' Return  : Sperrmethode: 0 = Lock-by-Select (Sperren bei Daten-
'                             satz-Selektion)
'                         1 = Lock-by-Edit (Sperren bei Beginn des
'                             Editiervorgangs eines Datensatzes)
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_Mode () As Integer

    Dim Lock_Mode As Integer

    glb_Last_Status = int_What_Lock_Mode(glb_Lock_File_Name, glb_User_Name, Lock_Mode, glb_Last_DOS_Error)
    
    Get_Lock_Mode = Lock_Mode

End Function

'============================================================
' Routine : Get_Lock_Seconds
'============================================================
' Aufgabe : Liefert die Sperrzeit fr einen gesperrten Datensatz
'           in Sekunden (wird ab dem Zeitpunkt der Sperrung ge-
'           rechnet). Ist der Datensatz nicht gesperrt oder
'           Fehler aufgetreten, so wird der Wert 0 zurckgelie-
'           fert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Sperrzeit in Sekunden
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_Seconds (Table_Name As String, Record_ID As Long) As Long

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer

    Get_Lock_Seconds = 0

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Select Case glb_Last_Status
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Get_Lock_Seconds = Lock_Seconds
    End Select

End Function

'============================================================
' Routine : Get_Lock_Time
'============================================================
' Aufgabe : Liefert die Sperrzeit eines Datensatzes in Form
'           eines formatierten Strings zurck, falls der Daten-
'           satz gesperrt ist. Ist der Datensatz nicht gesperrt
'           oder ein Fehler aufgetreten, so wird ein leerer
'           String zurckgeliefert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Sperrzeit; Format: "HH:MM:SS" bzw. leerer String
' Info    : Eine Sperrung liegt auch vor, wenn die evtl. ein-
'           gestellte Zeitmarke (Lock-Timeout) fr die automa-
'           tische Entsperrberwachung bereits berschritten
'           ist. Dies jedoch kann mit Hilfe der Funktion
'           Is_Record_Locked abgefragt werden.
'           Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_Time (Table_Name As String, Record_ID As Long) As String

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer
    Dim Str_Hour As String
    Dim Str_Min As String
    Dim Str_Sec As String

    Get_Lock_Time = ""

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Select Case glb_Last_Status
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Str_Hour = LTrim$(RTrim$(Str$(Lock_Hour)))
            If Lock_Hour < 10 Then Str_Hour = "0" & Str_Hour
            Str_Min = LTrim$(RTrim$(Str$(Lock_Min)))
            If Lock_Min < 10 Then Str_Min = "0" & Str_Min
            Str_Sec = LTrim$(RTrim$(Str$(Lock_Sec)))
            If Lock_Sec < 10 Then Str_Sec = "0" & Str_Sec
            Get_Lock_Time = Str_Hour & ":" & Str_Min & ":" & Str_Sec
    End Select

End Function

'============================================================
' Routine : Get_Lock_Timeout
'============================================================
' Aufgabe : Liefert die fr einen gesperrten Datensatz einge-
'           stellte Zeitmarke fr die automatische Entsperrber-
'           wachung zurck. Der Wert 0 bedeutet keine automati-
'           sche Entsperrberwachung. Ist der Datensatz nicht
'           gesperrt oder Fehler aufgetreten, so wird der Wert
'           -1 zurckgeliefert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Lock-Timeout in Sekunden
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_Timeout (Table_Name As String, Record_ID As Long) As Integer

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer

    Get_Lock_Timeout = -1

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Select Case glb_Last_Status
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Get_Lock_Timeout = Lock_Timeout
    End Select

End Function

'============================================================
' Routine : Get_Lock_User
'============================================================
' Aufgabe : Liefert den Namen des Benutzers zurck, der den an-
'           gegebenen Datensatz gesperrt hat. Ist der Datensatz
'           nicht gesperrt oder ein Fehler aufgetreten, so wird
'           ein leerer String zurckgeliefert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Benutzername (String) bzw. leerer String
' Info    : Eine Sperrung liegt auch vor, wenn die evtl. ein-
'           gestellte Zeitmarke (Lock-Timeout) fr die automa-
'           tische Entsperrberwachung bereits berschritten
'           ist. Dies jedoch kann mit Hilfe der Funktion
'           Is_Record_Locked abgefragt werden.
'           Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Lock_User (Table_Name As String, Record_ID As Long) As String

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer

    Get_Lock_User = ""

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Lock_User = LTrim$(RTrim$(Lock_User))
    If Lock_User <> "" Then
        Get_Lock_User = Lock_User
    End If

End Function

'============================================================
' Routine : Get_Login_Date
'============================================================
' Aufgabe : Liefert das Login-Datum des akt. Benutzers in Form
'           eines formatierten Strings. Ist der Benutzer nicht
'           im System eingeloggt oder ist ein Fehler aufgetreten,
'           so wird ein leerer String zurckgeliefert.
' Eingabe : keine
' Ausgabe : keine
' Return  : Login-Datum; Format: "DD.MM.YYYY" bzw. leerer String
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Login_Date () As String

    Dim Day_ As Integer
    Dim Month_ As Integer
    Dim Year_ As Integer
    Dim Hour_ As Integer
    Dim Min_ As Integer
    Dim Sec_ As Integer
    Dim Str_Day As String
    Dim Str_Month As String
    Dim Str_Year As String

    Get_Login_Date = ""

    glb_Last_Status = int_Login_DateTime(glb_Lock_File_Name, glb_User_Name, Day_, Month_, Year_, Hour_, Min_, Sec_, glb_Last_DOS_Error)

    If glb_Last_Status = st_No_Error Then
        Str_Day = LTrim$(RTrim$(Str$(Day_)))
        If Day_ < 10 Then Str_Day = "0" & Str_Day
        Str_Month = LTrim$(RTrim$(Str$(Month_)))
        If Month_ < 10 Then Str_Month = "0" & Str_Month
        Str_Year = LTrim$(RTrim$(Str$(Year_)))
        Get_Login_Date = Str_Day & "." & Str_Month & "." & Str_Year
    End If

End Function

'============================================================
' Routine : Get_Login_Seconds
'============================================================
' Aufgabe : Liefert die Login-Zeit des akt. Benutzers in Sekun-
'           den. Ist der Benutzer nicht im System eingeloggt
'           oder ist ein Fehler aufgetreten, so wird 0 zurck-
'           geliefert.
' Eingabe : keine
' Ausgabe : keine
' Return  : Login-Zeit in Sekunden
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Login_Seconds () As Long

    Dim Seconds As Long

    Get_Login_Seconds = 0

    glb_Last_Status = int_Login_Seconds(glb_Lock_File_Name, glb_User_Name, Seconds, glb_Last_DOS_Error)
    
    If glb_Last_Status = st_No_Error Then
        Get_Login_Seconds = Seconds
    End If

End Function

'============================================================
' Routine : Get_Login_Time
'============================================================
' Aufgabe : Liefert die Login-Zeit des akt. Benutzers in Form
'           eines formatierten Strings. Ist der Benutzer nicht
'           im System eingeloggt oder ist ein Fehler aufgetreten,
'           so wird ein leerer String zurckgeliefert.
' Eingabe : keine
' Ausgabe : keine
' Return  : Login-Zeit; Format: "HH:MM:SS" bzw. leerer String
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Get_Login_Time () As String

    Dim Day_ As Integer
    Dim Month_ As Integer
    Dim Year_ As Integer
    Dim Hour_ As Integer
    Dim Min_ As Integer
    Dim Sec_ As Integer
    Dim Str_Hour As String
    Dim Str_Min As String
    Dim Str_Sec As String

    Get_Login_Time = ""

    glb_Last_Status = int_Login_DateTime(glb_Lock_File_Name, glb_User_Name, Day_, Month_, Year_, Hour_, Min_, Sec_, glb_Last_DOS_Error)

    If glb_Last_Status = st_No_Error Then
        Str_Hour = LTrim$(RTrim$(Str$(Hour_)))
        If Hour_ < 10 Then Str_Hour = "0" & Str_Hour
        Str_Min = LTrim$(RTrim$(Str$(Min_)))
        If Min_ < 10 Then Str_Min = "0" & Str_Min
        Str_Sec = LTrim$(RTrim$(Str$(Sec_)))
        If Sec_ < 10 Then Str_Sec = "0" & Str_Sec
        Get_Login_Time = Str_Hour & ":" & Str_Min & ":" & Str_Sec
    End If

End Function

'============================================================
' Routine : Init_Locking
'============================================================
' Aufgabe : Initialisiert das Record-Locking-System.
' Eingabe : File_Name = Name der Record-Locking-Verwaltungsdatei
'           User_Name = Benutzername (max. 20 Zeichen)
'           Lock_Mode = Sperrmethode: 0 = Lock-by-Select (Sper-
'                                         ren bei Datensatz-Se-
'                                         lektion)
'                                     1 = Lock-by-Edit (Sperren
'                                         bei Beginn des Editier-
'                                         vorgangs eines Daten-
'                                         satzes)
' Ausgabe : keine
' Info    : File_Name, User_Name und Lock_Mode werden nach Auf-
'           ruf dieser Routine in entsprechenden globalen Va-
'           riablen gespeichert, die dann in anderen Routinen
'           benutzt werden. Auf diese Weise bleiben dem Prog-
'           rammierer Eingabeparameter erspart.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Sub Init_Locking (File_Name As String, User_Name As String, Lock_Mode As Integer)

    If Lock_Mode < lock_by_Select Then Lock_Mode = lock_by_Select
    If Lock_Mode > lock_by_Edit Then Lock_Mode = lock_by_Edit

    glb_Last_Status = int_Init_Locking(File_Name, User_Name, Lock_Mode, glb_Last_DOS_Error)
    
    If glb_Last_Status = st_No_Error Then
        glb_Lock_File_Name = File_Name
        glb_User_Name = User_Name
        glb_Lock_Mode = Lock_Mode
    End If

End Sub

'============================================================
' Routine : Is_Data_Changed
'============================================================
' Aufgabe : Liefert True zurck, wenn Daten innerhalb einer Form
'           gendert wurden (d.h. der Datensatz editiert wurde),
'           sonst False.
' Eingabe : F = Form
' Ausgabe : keine
' Return  : True bzw. False
'------------------------------------------------------------
'
Function Is_Data_Changed (F As Form) As Integer

    Dim i

    Is_Data_Changed = False

    If F.Controls.Count > 0 Then ' prfen, ob Form Steuerelemente enthlt
        For i = 0 To F.Controls.Count - 1 ' alle Steuerelemente durchlaufen
            If Is_Edit_Control(F.Controls(i)) Then ' prfen, ob es sich um ein Editierfeld handelt
                If F.Controls(i).DataChanged Then ' prfen, ob Daten gendert wurden
                    Is_Data_Changed = True
                    Exit Function ' Funktion verlassen
                End If
            End If
        Next i
    End If

End Function

'============================================================
' Routine : Is_Edit_Control
'============================================================
' Aufgabe : Liefert True zurck, wenn das angegebene Steuerelement
'           ein Editierfeld ist, sonst False.
' Eingabe : C = Steuerelement
' Ausgabe : keine
' Return  : True bzw. False
'------------------------------------------------------------
'
Function Is_Edit_Control (C As Control) As Integer

    Is_Edit_Control = False

    If TypeOf C Is TextBox Then ' Textfeld?
        Is_Edit_Control = True 'ist ein Eingabefeld
    ElseIf TypeOf C Is ComboBox Then ' Kombinationslistenfeld?
        Is_Edit_Control = True
    ElseIf TypeOf C Is CheckBox Then ' Kontrollfeld?
        Is_Edit_Control = True
    ElseIf TypeOf C Is OptionButton Then ' Optionsfeld?
        Is_Edit_Control = True
    End If

End Function

'============================================================
' Routine : Is_Record_Locked
'============================================================
' Aufgabe : Liefert st_Record_Not_Locked zurck, wenn der an-
'           gegebene Datensatz nicht gesperrt ist. Ist der Da-
'           tensatz gesperrt, so wird einer der folgenden Wer-
'           te zurckgeliefert:
'           st_Lock_By_Same, st_Lock_By_Others,
'           st_Lock_By_Same_Out, st_Lock_By_Others_Out
'           Die Bedeutung der Konstanten ist im Deklarations-
'           teil dieses Moduls erlutert.
'           Tritt ein Fehler auf, so wird der entsprechende
'           Fehlerstatus in der Variablen glb_Last_Status
'           gespeichert. Handelt es sich sogar um einen DOS-
'           Fehler, so wird dessen Code in der globalen Va-
'           riablen glb_Last_DOS_Error vermerkt.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : Status (wie oben beschrieben)
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Function Is_Record_Locked (Table_Name As String, Record_ID As Long) As Integer

    Dim Lock_User As String
    Dim Lock_Day As Integer
    Dim Lock_Month As Integer
    Dim Lock_Year As Integer
    Dim Lock_Hour As Integer
    Dim Lock_Min As Integer
    Dim Lock_Sec As Integer
    Dim Lock_Seconds As Long
    Dim Lock_Timeout As Integer

    Is_Record_Locked = st_Record_Not_Locked

    Lock_User = String$(255, 32)

    glb_Last_Status = int_Is_Record_Locked(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Lock_User, Lock_Day, Lock_Month, Lock_Year, Lock_Hour, Lock_Min, Lock_Sec, Lock_Seconds, Lock_Timeout, glb_Last_DOS_Error)

    Select Case glb_Last_Status
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Is_Record_Locked = glb_Last_Status
    End Select

End Function

'============================================================
' Routine : Is_Record_Locked_Gen
'============================================================
' Aufgabe : Prft, ob ein Datensatz generell gesperrt ist, d.h.
'           unabhngig davon, welcher Benutzer ihn gesperrt
'           hat und ob die Zeitmarke fr die evtl. eingestell-
'           te Entsperrberwachung bereits berschritten ist
'           oder nicht. Es wird True bzw. False (Datensatz nicht
'           gesperrt) zurckgeliefert.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Return  : True bzw. False
'------------------------------------------------------------
'
Function Is_Record_Locked_Gen (Table_Name As String, Record_ID As Long) As Integer

    Dim Result As Integer

    Is_Record_Locked_Gen = False

    Result = Is_Record_Locked(Table_Name, Record_ID)

    Select Case Result
        Case st_Lock_By_Same, st_Lock_By_Others, st_Lock_By_Same_Out, st_Lock_By_Others_Out
            Is_Record_Locked_Gen = True
    End Select

End Function

'============================================================
' Routine : Lock_Edit_Controls
'============================================================
' Aufgabe : Sperrt alle Editierfelder innerhalb einer Form.
' Eingabe : F = Form
' Ausgabe : keine
'------------------------------------------------------------
'
Sub Lock_Edit_Controls (F As Form)

    Dim i

    If F.Controls.Count > 0 Then ' prfen, ob Form Steuerelemente enthlt
        For i = 0 To F.Controls.Count - 1 ' alle Steuerelemente durchlaufen
            If Is_Edit_Control(F.Controls(i)) Then ' prfen, ob es sich um ein Editierfeld handelt
                F.Controls(i).Enabled = False ' Feld deaktivieren
            End If
        Next i
    End If

End Sub

'============================================================
' Routine : Lock_Record
'============================================================
' Aufgabe : Sperrt einen Datensatz in einer Datenbank-Tabelle
'           bzw. Abfrage. Ist der zu sperrende Datensatz bereits
'           gesperrt und ist fr den Datensatz eine automati-
'           sche Entsperrberwachung eingeschaltet, so wird ge-
'           prft, ob die eingestellte Zeitmarke berschritten
'           ist; ggf. wird in der Variablen glb_Last_Status ver-
'           merkt, da der Datensatz zu entsperren ist
'           (st_Lock_By_Same_Out, st_Lock_By_Others_Out). Ist
'           der Datensatz gesperrt und entweder keine Entsperr-
'           berwachung eingestellt oder die Zeitmarke noch
'           nicht berschritten, so hat die Variable glb_Last_Status
'           den Wert st_Lock_By_Same bzw. st_Lock_By_Others.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz_ID (mu >= 0 sein)
'           Timeout    = Zeitvorgabe fr die automatische Ent-
'                        sperrberwachung (in Sekunden; 0 bedeu-
'                        tet keine automatische Entsperrberwa-
'                        chung)
' Ausgabe : keine
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion werden die globalen Variablen glb_Lock_File_Name
'           und glb_User_Name benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Sub Lock_Record (Table_Name As String, Record_ID As Long, Timeout As Integer)

    glb_Last_Status = int_Lock_Record(glb_Lock_File_Name, glb_User_Name, Table_Name, Record_ID, Timeout, glb_Last_DOS_Error)

End Sub

'============================================================
' Routine : Unlock_Edit_Controls
'============================================================
' Aufgabe : Entsperrt alle Editierfelder innerhalb einer Form.
' Eingabe : F = Form
' Ausgabe : keine
'------------------------------------------------------------
'
Sub Unlock_Edit_Controls (F As Form)

    Dim i

    If F.Controls.Count > 0 Then ' prfen, ob Form Steuerelemente enthlt
        For i = 0 To F.Controls.Count - 1 ' alle Steuerelemente durchlaufen
            If Is_Edit_Control(F.Controls(i)) Then ' prfen, ob es sich um ein Editierfeld handelt
                F.Controls(i).Enabled = True ' Feld aktivieren
            End If
        Next i
    End If

End Sub

'============================================================
' Routine : Unlock_Record
'============================================================
' Aufgabe : Entsperrt den angegebenen Datensatz unabhngig da-
'           von, welcher Benutzer ihn gesperrt hat.
' Eingabe : Table_Name = Name der Datenbank-Tabelle bzw. Abfrage
'                        (max. 40 Zeichen)
'           Record_ID  = Datensatz-ID (mu >= 0 sein)
' Ausgabe : keine
' Info    : Fr den internen Aufruf der entsprechenden DLL-Funk-
'           tion wird die globale Variable glb_Lock_File_Name
'           benutzt.
'           Status und evtl. DOS-Fehler knnen ber die globalen
'           Variablen glb_Last_Status und glb_Last_DOS_Error
'           abgefragt werden.
'------------------------------------------------------------
'
Sub Unlock_Record (Table_Name As String, Record_ID As Long)

    glb_Last_Status = int_Unlock_Record(glb_Lock_File_Name, Table_Name, Record_ID, glb_Last_DOS_Error)

End Sub

